﻿namespace FakeItEasy.Core
{
    using System.Linq;
    using System.Reflection;

    internal static class AssemblyExtensions
    {
#if !SILVERLIGHT
        private static readonly string FakeItEasyAssemblyName = Assembly.GetExecutingAssembly().FullName;

        /// <summary>
        /// Determines whether an assembly references FakeItEasy.
        /// </summary>
        /// <param name="assembly">The assembly to check.</param>
        /// <returns>Whether or not the assembly references FakeItEasy.</returns>
        public static bool ReferencesFakeItEasy(this Assembly assembly)
        {
            Guard.AgainstNull(assembly, "assembly");

            return assembly.GetReferencedAssemblies().Any(r => r.FullName == FakeItEasyAssemblyName);
        }
#endif

        /// <summary>
        /// Gets the simple name of the assembly. Can be used in Silverlight solutions.
        /// </summary>
        /// <param name="assembly">The assembly.</param>
        /// <returns>The simple name of the assembly.</returns>
        public static string Name(this Assembly assembly)
        {
            return new AssemblyName(assembly.FullName).Name;
        }
    }
}
